#include "recorderdatabase.h"

RecorderDataBase::RecorderDataBase(QObject *parent) : QObject(parent)
{
    qDebug() << QSqlDatabase::drivers(); //当前环境支持哪些数据库
    m_dataBase = QSqlDatabase::addDatabase("QSQLITE");
    QString dirPath = QString(getenv("HOME")) + "/.config/.kylin_recorder_Flag.db";
    QTextCodec::setCodecForLocale(QTextCodec::codecForLocale()); //设置显示中文
    m_dataBase.setDatabaseName(dirPath);
    connect(this, &RecorderDataBase::insertDataSignal, this, &RecorderDataBase::insertDataSlot);
    connect(this, &RecorderDataBase::deleteDataSignal, this, &RecorderDataBase::deleteDataSlot);
    connect(this, &RecorderDataBase::searchDataSignal, this, &RecorderDataBase::searchDataSlot);
    connect(this, &RecorderDataBase::addRecordSignal, this, &RecorderDataBase::addNewFile);
    connect(this, &RecorderDataBase::deleteRecordSignal,this, &RecorderDataBase::deleteFile);
}

int RecorderDataBase::getStatus()
{
    return m_dbStatus;
}

QStringList RecorderDataBase::getDataList()
{
    return searchDataList;
}

int RecorderDataBase::initDataBase()
{
    if (!m_dataBase.open()) { //如果数据库打开失败，会弹出一个警告窗口;由于此类非指针类型因此是0
        QMessageBox::warning(0, QObject::tr("Database Error"), m_dataBase.lastError().text());
        m_dbStatus = DB_UNCONNECT;
        return DB_UNCONNECT;
    } else {
        QSqlQuery query;
        QString creatTableStr = "CREATE TABLE IF NOT EXISTS Flag  "
                                "(file_id     integer   PRIMARY KEY AUTOINCREMENT,  "
                                "file_name    char(50)  NOT NULL,  "
                                "flag_time     char(50)  NOT NULL); ";
        query.prepare(creatTableStr);
        if (!query.exec()) {
            qDebug() << "建表失败!";
            m_dbStatus = CREATE_TABLE_FAILED;
            return CREATE_TABLE_FAILED;
        } else {
            qDebug() << "建表成功!";
            m_dbStatus = DB_OP_SUCC;
        }

        //新建第二个表
        QSqlQuery query2;
        QString creatTableStr2 = "CREATE TABLE IF NOT EXISTS File  "
                                "(file_id     integer   PRIMARY KEY AUTOINCREMENT,  "
                                "file_name    char(50)  NOT NULL,  "
                                "create_time  char(50)  NOT NULL); ";
        query2.prepare(creatTableStr2);
        if (!query2.exec()) {
            qDebug() << "建表失败!";
            m_dbStatus = CREATE_TABLE_FAILED;
            return CREATE_TABLE_FAILED;
        } else {
            qDebug() << "建表成功!";
            m_dbStatus = DB_OP_SUCC;
        }
    }
    return m_dbStatus;
}

int RecorderDataBase::addNewFile(QString name,QString time)
{
    if (!m_dataBase.open()) { //如果数据库打开失败，会弹出一个警告窗口;由于此类非指针类型因此是0

        QMessageBox::warning(0, QObject::tr("Database Error"), m_dataBase.lastError().text());
        m_dbStatus = DB_UNCONNECT;
        return DB_UNCONNECT;

    } else {

        int num = 0;
        QString searchCmd = QString("select count(file_name) from File where file_name = '%1' AND create_time = '%2'")
                                .arg(name)
                                .arg(time);
        QSqlQuery searchQuery;
        searchQuery.exec(searchCmd);
        if (!searchQuery.exec(searchCmd)) {
            qDebug() << "query error :" << searchQuery.lastError();
            m_dbStatus = DB_OP_GET_FAILED;
            return DB_OP_GET_FAILED;
        } else {
            while (searchQuery.next()) {
                num = searchQuery.value(0).toInt();
            }
        }

        if (num > 0) {
            qDebug() << "不可重复添加:" << DB_OP_ADD_REPEAT;
            m_dbStatus = DB_OP_ADD_REPEAT;
            return DB_OP_ADD_REPEAT;

        } else {

            QString cmd =
                QString("insert into File (%1) values('%2','%3')").arg(ITEM_FIELD2).arg(name).arg(time);
            QSqlQuery query;
            if (!query.exec(cmd)) {
                qDebug() << "query error :" << query.lastError();
                m_dbStatus = DB_OP_ADD_FAILED;
                return DB_OP_ADD_FAILED;
            } else {
                qDebug() << "插入成功!";
                m_dbStatus = DB_OP_SUCC;
                return DB_OP_SUCC;
            }
        }
    }
}
int RecorderDataBase::deleteFile(QString name)
{
    QString existBaseName;
     if (!m_dataBase.open()) { //如果数据库打开失败，会弹出一个警告窗口;由于此类非指针类型因此是0
         QMessageBox::warning(0, QObject::tr("Database Error"), m_dataBase.lastError().text());
         m_dbStatus = DB_UNCONNECT;
         return DB_UNCONNECT;
     } else {
         QString searchCmd = QString("select file_name from File where file_name = '%1'").arg(name);
         QSqlQuery searchQuery;
         searchQuery.exec(searchCmd);
         if (!searchQuery.exec(searchCmd)) {
             qDebug() << "query error :" << searchQuery.lastError();
             m_dbStatus = DB_OP_GET_FAILED;
             return DB_OP_GET_FAILED;
         } else {
             while (searchQuery.next()) {
                 existBaseName = searchQuery.value(0).toString();
             }
         }
         if (existBaseName == name) {
             QString cmd = QString("delete from Flag where file_name = '%1'").arg(name);
             QSqlQuery query;
             if (!query.exec(cmd)) {
                 qDebug() << "query error :" << query.lastError();
                 m_dbStatus = DB_OP_DEL_FAILED;
                 return DB_OP_DEL_FAILED;
             } else {
                 qDebug() << "删除成功!";
                 m_dbStatus = DB_OP_SUCC;
                 return DB_OP_SUCC;
             }

         } else {
             m_dbStatus = FLAG_UNFOUND;
             return FLAG_UNFOUND;
         }
     }
}

void RecorderDataBase::getAllFiles(QStringList &fileNames, QStringList &fileTimes)
{
    fileNames.clear();
    fileTimes.clear();
    if (!m_dataBase.open()) { //如果数据库打开失败，会弹出一个警告窗口;由于此类非指针类型因此是0
        QMessageBox::warning(0, QObject::tr("Database Error"), m_dataBase.lastError().text());
        m_dbStatus = DB_UNCONNECT;
        return ;
    }

    QString query = QString("select %1,%2 from File").arg("file_name").arg("create_time");
    QSqlQuery searchQuery;
    searchQuery.exec(query);
    if (!searchQuery.exec(query)) {
        qDebug() << "query error :" << searchQuery.lastError();
        m_dbStatus = DB_OP_GET_FAILED;
        return;
    }

    while (searchQuery.next()) {
        QString name = searchQuery.value(0).toString();
        QString time = searchQuery.value(1).toString();
        fileNames.append(name);
        fileTimes.append(time);
    }
    m_dbStatus = DB_OP_SUCC;
    return ;
}

int RecorderDataBase::insertDataSlot(QString fileName, QString flagTime)
{
    if (!m_dataBase.open()) { //如果数据库打开失败，会弹出一个警告窗口;由于此类非指针类型因此是0

        QMessageBox::warning(0, QObject::tr("Database Error"), m_dataBase.lastError().text());
        m_dbStatus = DB_UNCONNECT;
        return DB_UNCONNECT;

    } else {

        int num = 0;
        QString searchCmd = QString("select count(flag_time) from Flag where file_name = '%1' AND flag_time = '%2'")
                                .arg(fileName)
                                .arg(flagTime);
        QSqlQuery searchQuery;
        searchQuery.exec(searchCmd);
        if (!searchQuery.exec(searchCmd)) {
            qDebug() << "query error :" << searchQuery.lastError();
            m_dbStatus = DB_OP_GET_FAILED;
            return DB_OP_GET_FAILED;

        } else {

            while (searchQuery.next()) {

                num = searchQuery.value(0).toInt();
            }
        }
        if (num > 0) {
            qDebug() << "不可重复添加:" << DB_OP_ADD_REPEAT;
            m_dbStatus = DB_OP_ADD_REPEAT;
            return DB_OP_ADD_REPEAT;

        } else {

            QString cmd =
                QString("insert into Flag (%1) values('%2','%3')").arg(ITEM_FIELD).arg(fileName).arg(flagTime);
            QSqlQuery query;
            if (!query.exec(cmd)) {
                qDebug() << "query error :" << query.lastError();
                m_dbStatus = DB_OP_ADD_FAILED;
                return DB_OP_ADD_FAILED;
            } else {
                qDebug() << "插入成功!";
                m_dbStatus = DB_OP_SUCC;
                return DB_OP_SUCC;
            }
        }
    }
}
int RecorderDataBase::deleteDataSlot(QString fileBaseName)
{
    qDebug() << "删除数据库中此文件的标记" << fileBaseName;
    QString existBaseName;
    if (!m_dataBase.open()) { //如果数据库打开失败，会弹出一个警告窗口;由于此类非指针类型因此是0

        QMessageBox::warning(0, QObject::tr("Database Error"), m_dataBase.lastError().text());
        m_dbStatus = DB_UNCONNECT;
        return DB_UNCONNECT;

    } else {
        QString searchCmd = QString("select file_name from Flag where file_name = '%1'").arg(fileBaseName);

        QSqlQuery searchQuery;
        searchQuery.exec(searchCmd);
        if (!searchQuery.exec(searchCmd)) {
            qDebug() << "query error :" << searchQuery.lastError();
            m_dbStatus = DB_OP_GET_FAILED;
            return DB_OP_GET_FAILED;

        } else {

            while (searchQuery.next()) {

                existBaseName = searchQuery.value(0).toString();
            }
        }
        if (existBaseName == fileBaseName) {

            QString cmd = QString("delete from Flag where file_name = '%1'").arg(fileBaseName);
            QSqlQuery query;
            if (!query.exec(cmd)) {
                qDebug() << "query error :" << query.lastError();
                m_dbStatus = DB_OP_DEL_FAILED;
                return DB_OP_DEL_FAILED;
            } else {
                qDebug() << "删除成功!";
                m_dbStatus = DB_OP_SUCC;
                return DB_OP_SUCC;
            }

        } else {

            m_dbStatus = FLAG_UNFOUND;
            return FLAG_UNFOUND;
        }
    }
}
int RecorderDataBase::searchDataSlot(QString fileBaseName)
{
    qDebug() << "查找标记的文件 :" << fileBaseName;
    QString flagTime;
    if (!m_dataBase.open()) { //如果数据库打开失败，会弹出一个警告窗口;由于此类非指针类型因此是0

        QMessageBox::warning(0, QObject::tr("Database Error"), m_dataBase.lastError().text());
        m_dbStatus = DB_UNCONNECT;
        return DB_UNCONNECT;

    } else {
        QString searchCmd = QString("select flag_time from Flag where file_name = '%1'").arg(fileBaseName);

        QSqlQuery searchQuery;
        searchQuery.exec(searchCmd);
        if (!searchQuery.exec(searchCmd)) {
            qDebug() << "query error :" << searchQuery.lastError();
            m_dbStatus = DB_OP_GET_FAILED;
            return DB_OP_GET_FAILED;

        } else {
            searchDataList.clear();
            while (searchQuery.next()) {

                flagTime = searchQuery.value(0).toString();
                searchDataList.append(flagTime);
            }
            m_dbStatus = DB_OP_SUCC;
            return DB_OP_SUCC;
        }
    }
}

RecorderDataBase::~RecorderDataBase()
{
    qDebug() << "析构RecorderDataBase";
    if (m_dataBase.open()) {
        m_dataBase.close();
    }
}
